//
//  CDAnalyticsListener.java
//
//  Copyright 2018 Localytics. All rights reserved.
//

package com.localytics.phonegap;

import android.content.Intent;

import androidx.annotation.NonNull;

import com.localytics.androidx.CallToActionListenerV2;
import com.localytics.androidx.Campaign;
import com.localytics.androidx.InAppCampaign;
import com.localytics.androidx.InboxCampaign;
import com.localytics.androidx.PlacesCampaign;
import com.localytics.androidx.PushCampaign;

import org.apache.cordova.CallbackContext;
import org.apache.cordova.PluginResult;
import org.json.JSONException;
import org.json.JSONObject;

public class CDCTAListener implements CallToActionListenerV2 {

    private final CallbackContext callbackContext;

    public CDCTAListener(CallbackContext callbackContext) {
        this.callbackContext = callbackContext;
    }


    @Override
    public boolean localyticsShouldDeeplink(@NonNull String url, @NonNull Campaign campaign) {
        JSONObject object = new JSONObject();
        try {
            object.put("method", "localyticsShouldDeeplink");

            JSONObject params = new JSONObject();
            params.put("url", url);
            params.put("campaign", getJSONFromGenericCampaign(campaign));
        } catch (JSONException e) {
            // ignore
        }

        PluginResult result = new PluginResult(PluginResult.Status.OK, object);
        result.setKeepCallback(true);
        callbackContext.sendPluginResult(result);
        return true;
    }

    @Override
    public void localyticsDidOptOut(boolean optOut, @NonNull Campaign campaign) {
        JSONObject object = new JSONObject();
        try {
            object.put("method", "localyticsDidOptOut");

            JSONObject params = new JSONObject();
            params.put("optedOut", optOut);
            params.put("campaign", getJSONFromGenericCampaign(campaign));
        } catch (JSONException e) {
            // ignore
        }

        PluginResult result = new PluginResult(PluginResult.Status.OK, object);
        result.setKeepCallback(true);
        callbackContext.sendPluginResult(result);
    }

    @Override
    public void localyticsDidPrivacyOptOut(boolean optOut, @NonNull Campaign campaign) {
        JSONObject object = new JSONObject();
        try {
            object.put("method", "localyticsDidPrivacyOptOut");

            JSONObject params = new JSONObject();
            params.put("privacyOptedOut", optOut);
            params.put("campaign", getJSONFromGenericCampaign(campaign));
        } catch (JSONException e) {
            // ignore
        }

        PluginResult result = new PluginResult(PluginResult.Status.OK, object);
        result.setKeepCallback(true);
        callbackContext.sendPluginResult(result);
    }

    @Override
    public boolean localyticsShouldPromptForLocationPermissions(@NonNull Campaign campaign) {
        JSONObject object = new JSONObject();
        try {
            object.put("method", "localyticsShouldPromptForLocationPermissions");

            JSONObject params = new JSONObject();
            params.put("campaign", getJSONFromGenericCampaign(campaign));
        } catch (JSONException e) {
            // ignore
        }

        PluginResult result = new PluginResult(PluginResult.Status.OK, object);
        result.setKeepCallback(true);
        callbackContext.sendPluginResult(result);
        return true;
    }

    @Override
    public boolean localyticsShouldPromptForNotificationPermissions(@NonNull Campaign campaign) {
        JSONObject object = new JSONObject();
        try {
            object.put("method", "localyticsShouldPromptForNotificationPermissions");

            JSONObject params = new JSONObject();
            params.put("campaign", getJSONFromGenericCampaign(campaign));
        } catch (JSONException e) {
            // ignore
        }

        PluginResult result = new PluginResult(PluginResult.Status.OK, object);
        result.setKeepCallback(true);
        callbackContext.sendPluginResult(result);
        return true;
    }

    @Override
    public boolean localyticsShouldDeeplinkToSettings(Intent intent, Campaign campaign) {
        JSONObject object = new JSONObject();
        try {
            object.put("method", "localyticsShouldDeeplinkToSettings");

            JSONObject params = new JSONObject();
            params.put("campaign", getJSONFromGenericCampaign(campaign));
        } catch (JSONException e) {
            // ignore
        }
        PluginResult result = new PluginResult(PluginResult.Status.OK, object);
        result.setKeepCallback(true);
        callbackContext.sendPluginResult(result);
        return true;
    }

    private JSONObject getJSONFromGenericCampaign(Campaign campaign) throws JSONException {
        if (campaign instanceof PlacesCampaign) {
            return LocalyticsPlugin.toPlacesJSON((PlacesCampaign) campaign);
        } else if (campaign instanceof InboxCampaign) {
            return LocalyticsPlugin.toInboxJSON((InboxCampaign) campaign);
        } else if (campaign instanceof InAppCampaign) {
            return LocalyticsPlugin.toInAppJSON((InAppCampaign) campaign);
        } else if (campaign instanceof PushCampaign) {
            return LocalyticsPlugin.toPushJSON((PushCampaign) campaign);
        }
        return null; //should never happen.

    }
}
